function object = flt_eval( cmd, varargin )

% object = flt_eval( cmd, input1, [input2, ...] )
% object = flt_eval( {channels, constructor, cmd}, input1, [input2, ...] )
%
% This filter creates its output by reading new values from the inputs,
% placing them into cell array 'in', each input vector into its own cell,
% and finally evaluating the given command to get the filter output.
%
% The command should always return a constant length vector and should
% tolerate NaN values in the inputs (meaning 'no input').
%
% The first form creates a stateless evaluator. The return value of the command
% string is used as the filter output. The given command string should not
% contain multiple matlab commands (i.e., it should not contain commands
% separated by a semicolon).
%
% The second form creates a stateful evaluator. The command string can (and
% usually will) contain multiple matlab commands separated by a semicolon. The
% variable 'state' can be used in the command to store an internal state: it
% should be initialized in the 'constructor' command, and it will maintain its
% contents between evaluations. The filter output should be assigned to the
% variable 'out' in the command. The command may create any temporary variables
% except the reserved variable 'this'. Output channel count must be provided in
% this call form.
%
% If all outputs returned by the command are NaNs, then all inputs are flushed
% (so that they all return only NaNs).
%
% Examples:
%   filter = flt_eval( '(in{1} + in{2}) ./ in{3}', input1, input2, input3 );
%   filter = flt_eval( { 1, 'state = 0.0;', ...
%                        [ 'out = sin(state)*in{1} + cos(state)*in{2}; ' ...
%                          'state = state + pi/8;' ] }, ...
%                      input1, input2 );


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% define the class so that matlab recognizes it when loading objects
if nargin == 0
  this.enabled       = [];
  this.nchannels     = [];
  this.ninputs       = [];
  this.constructor   = [];
  this.cmd           = [];
  this.state         = [];
  this.inputs        = [];
  this.nan_vals      = [];
  object = class( this, 'flt_eval', flt );
  return;
end


if isa(cmd, 'cell')
  [nchannels, constructor, cmd] = deal( cmd{:} );
else
  nchannels   = test_output_nchannels(cmd, varargin);
  constructor = [];
  cmd         = cmd;
end


this.enabled       = 0;
this.nchannels     = nchannels;
this.ninputs       = length(varargin);
this.constructor   = constructor;
this.cmd           = cmd;
this.state         = [];
this.inputs        = varargin;
this.nan_vals      = repmat( NaN, 1, this.nchannels );


object = class( this, 'flt_eval', flt( this.nchannels ) );
return;








function nchannels = test_output_nchannels( cmd, inputs )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:length(inputs)
  in{i} = NaN * zeros( 1, get_nchannels( inputs{i} ) );
end

nchannels = length( eval(cmd) );
return;
